from virgotools import *
import numpy as np
import pytest
import sys
import multiprocessing as mp
import time
import logging

logger = mp.get_logger()
logger.setLevel(logging.INFO)

queue = mp.Queue()

try:
    # python 3.4+ should use builtin unittest.mock not mock package
    from unittest.mock import patch
except ImportError:
    from mock import patch

testargs = ["test_server.py","TestServer.cfg"]
with patch.object(sys,'argv',testargs):
	from pyserver import *

step = 0
counter = 0

@cm_handler
def handle_step(message, sender, _):
    global step
    sender = pvt_utils.native_str(sender)
    logger.info('Received STEP message from %s', sender)
    try:
        message = cm_lib.parse_cm_message(message)
        logger.info('Parsed message: %r', message)
        string = pvt_utils.native_str(message[0])
        logger.info('String parameter: %r', string)
        step = int(string)
    except:  # catch any error, don't want to crash due to bad message
        logger.exception('Error while processing cm message')
    return cm.CmMessageOk  # or cm.CmMessageOk


def worker():
    global step, counter, connection
    counter = counter + step
    if(counter>5):
    	queue.put(True)
    fdata = "counter " + str(counter)
    return fdata

def init():
    global step, connection
    logger.warning("Doing init")
    cm.CmMessageInstallHandler(handle_step, 'STEP')
    step = 1
    return None

def cleanup():
    global step
    logger.warning("Doing cleanup")
    return None


def test_run():
	#Setup a dummy sys.argv to be used when running the process
	testargs = ["test_server.py","TestServer.cfg"]
	with patch.object(sys,'argv',testargs):

		#Run the server in a process so it can be stopped using CfgExit
		process = mp.Process(target=run_server,args=(worker,),kwargs={"init":init,"cleanup":cleanup})

		#Start the process and mark the startime
		process.start()
		starttime = time.time()

		fail=False

		#Monitor the queue to see if it is populated
		while queue.empty():

			#Constantly check if process is alive
			if process.is_alive()==False:
				fail = True
				break

			#Check if we have gone beyond the time limit
			if(time.time()-starttime>30):
				fail = True
				break

		#If the process failed then raise as systemexit
		if fail:
			raise SystemExit("Server Exited Abnormally")
		else:
			#Stop the Server
			logger.info("Sending CfgExit to Server")
			cm_start("TestServerStopper")
			cm_send("TestServer","CfgExit")


